﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;

using EPiServer.Security;
using EPiServer.Web.Hosting;

namespace Nansen.Viddler.Hosting
{
    public class ViddlerDirectory : UnifiedDirectory
    {
        public ViddlerDirectory(VirtualPathUnifiedProvider provider, string virtualPath, string customSummaryVirtualPath, bool bypassAccessCheck) : base(provider, virtualPath, customSummaryVirtualPath, bypassAccessCheck)
        {
        }

        public override UnifiedFile CreateFile(string name)
        {
            return new ViddlerFile(this, this.Provider, string.Format("{0}/{1}", this.VirtualPath, name), this.Provider.BypassAccessCheck);
        }

        public override UnifiedDirectory Parent
        {
            get
            {
                string directory = VirtualPathUtility.GetDirectory(this.VirtualPath);
                
                if (this.BypassAccessCheck)
                {
                    return (VirtualPathHandler.Instance.GetDirectory(directory, true) as UnifiedDirectory);
                }

                return (base.Provider.GetDirectory(directory) as UnifiedDirectory);
            }
        }

        public override IEnumerable Directories
        {
            get
            {
                return new List<ViddlerDirectory>();
            }
        }

        public override System.Collections.IEnumerable Files
        {
            get
            {
                var viddler = new Viddler();
                IEnumerable<ViddlerVideo> videos = viddler.GetVideos();
                foreach(ViddlerVideo video in videos)
                {
                    var parent = new ViddlerDirectory(this.Provider, VirtualPathUtility.GetDirectory(this.VirtualPath), this.Provider.ConfigFileSummaryFullPath, this.Provider.BypassAccessCheck);

                    yield return new ViddlerFile(parent, this.Provider, this.VirtualPath + video.Title, this.Provider.BypassAccessCheck);
                }
            }
        }

        public override AccessControlList ACL
        {
            get
            {
                var acl = new AccessControlList();
                if (this.Provider.BypassAccessCheck)
                {
                    acl.Add(new AccessControlEntry("Everyone", AccessLevel.FullAccess, SecurityEntityType.Role));
                }

                return acl;
            }
        }

        public override IEnumerable Children
        {
            get
            {
                yield return this.Files;
                yield return this.Directories;
            }
        }

        public override UnifiedDirectory[] GetDirectories()
        {
            return this.Directories.Cast<UnifiedDirectory>().ToArray();
        }

        public override UnifiedFile[] GetFiles()
        {
            return this.Files.Cast<UnifiedFile>().ToArray();
        }
    }
}