﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Web;
using System.Net;

using EPiServer.Security;
using EPiServer.Web.Hosting;

namespace Nansen.Viddler.Hosting
{
    public class ViddlerFile : UnifiedFile
    {
        public ViddlerFile(UnifiedDirectory directory, VirtualPathUnifiedProvider provider, string virtualPath, bool bypassAccessCheck) : base(directory, provider, virtualPath, bypassAccessCheck)
        {
        }

        public string PlayerUrl
        {
            get
            {
                return string.Format("http://www.viddler.com/player/{0}/", this.Video.Id);
            }

        }

        private ViddlerVideo _video;
        public ViddlerVideo Video
        {
            get
            {
                if (_video == null)
                {
                    var viddler = new Viddler();

                    IEnumerable<ViddlerVideo> videoList = viddler.GetVideos();

                    int startIndex = this.VirtualPath.LastIndexOf('/');
                    string videoName = this.VirtualPath.Substring(startIndex + 1);

                    videoName = HttpUtility.UrlDecode(videoName);

                    List<ViddlerVideo> videos = videoList.ToList().Where(v => v.Title == videoName).ToList();

                    _video = videos.Count > 0 ? videos.Single() : null;
                }

                return _video;
            }
        }

        public override IUnifiedSummary Summary
        {
            get
            {
                return new ViddlerUnifiedSummary(this);
            }
        }

        // TODO: Lägg till stöd för icke-publika filmer
        public override AccessLevel QueryAccess()
        {
            return AccessLevel.FullAccess;
        }

        public override UnifiedDirectory Parent
        {
            get
            {
                return new ViddlerDirectory(this.Provider, this.VirtualPath, this.Provider.ConfigFileSummaryFullPath, this.Provider.BypassAccessCheck);
            }
        }

        public override void Delete()
        {
            var viddler = new Viddler();
            viddler.Delete(this.Video.Id);
        }

        public override string Extension
        {
            get
            {
                return "mpg";
            }
        }

        public override long Length
        {
            get
            {
                return 0;
            }
        }

        public override DateTime Created
        {
            get
            {
                return this.Video.Uploaded;
            }
        }

        public override DateTime Changed
        {
            get
            {
                return this.Video.Updated != DateTime.MinValue ? this.Video.Updated : this.Video.Uploaded;
            }
        }

        public override string Name
        {
            get
            {
                return this.Video.Title;
            }
        }

        public override Stream Open(FileMode mode, FileAccess access)
        {
            if (mode == FileMode.Create || mode == FileMode.CreateNew)
            {
                int startIndex = this.VirtualPath.LastIndexOf('/');
                string videoName = this.VirtualPath.Substring(startIndex + 1);

                videoName = HttpUtility.UrlDecode(videoName);

                var viddler = new Viddler();

                // fulhack
                string title = string.Empty;
                string description = string.Empty;
                string tags = string.Empty;

                foreach (string key in System.Web.HttpContext.Current.Request.Form.AllKeys)
                {
                    if (key.Contains("ViddlerTitle"))
                    {
                        title = System.Web.HttpContext.Current.Request.Form[key];
                    }
                    else if (key.Contains("ViddlerDescription"))
                    {
                        description = System.Web.HttpContext.Current.Request.Form[key];
                    }
                    else if (key.Contains("ViddlerTags"))
                    {
                        tags = System.Web.HttpContext.Current.Request.Form[key];
                    }
                }

                viddler.Upload(title, tags, description, HttpContext.Current.Request.Files[0].InputStream);
            }

            return HttpContext.Current.Request.Files[0].InputStream;
        }

        public override string PermanentLinkVirtualPath
        {
            get
            {
                return string.Format("~/viddler/{0}", this.Video.Id);
            }
        }

        public override string LocalPath
        {
            get
            {
                return this.Video.Url;
            }
        }

        public override bool IsDirectory
        {
            get
            {
                return false;
            }
        }
    }
}
