﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Text;
using System.Web;

namespace Nansen.Viddler.Hosting
{
    public class ViddlerVirtualPathProvider : EPiServer.Web.Hosting.VirtualPathUnifiedProvider
    {
        public ViddlerVirtualPathProvider(string name, NameValueCollection configParameters) : base(name, configParameters)
        {
            this.VirtualPathRoot = this.ConfigurationParameters["virtualPath"];
            this.ShowInFileManager = bool.Parse(this.ConfigurationParameters["showInFileManager"]);
            this.BypassAccessCheck = bool.Parse(this.ConfigurationParameters["bypassAccessCheck"]);
            this.ConfigFileSummaryFullPath = this.ConfigurationParameters["customFileSummaryPath"];
        }

        public new bool IsVirtualPath(string virtualPath)
        {
            return false;
        }

        public override System.Web.Hosting.VirtualDirectory GetDirectory(string virtualPath)
        {
            if (virtualPath.StartsWith(this.VirtualPathRoot))
            {
                var directory = new ViddlerDirectory(this, virtualPath, this.ConfigFileSummaryFullPath, this.BypassAccessCheck);
                return directory;
            }

            return this.Previous.GetDirectory(virtualPath);
        }

        public override System.Web.Hosting.VirtualFile GetFile(string virtualPath)
        {
            if (virtualPath.StartsWith(this.VirtualPathRoot))
            {
                var parent = new ViddlerDirectory(this, VirtualPathUtility.GetDirectory(virtualPath), this.ConfigFileSummaryFullPath, this.BypassAccessCheck);

                // fulhack... sätts i Viddler.Upload()
                // episerver vill ha null i HandleUploadedFiles() för att inte fastna i en while-loop, men inte några kodrader senare.
                if (HttpContext.Current.Items["tempViddlerVideoId"] != null)
                {
                    return new ViddlerTempFile(parent, this, virtualPath, this.BypassAccessCheck);
                }

                var viddlerFile = new ViddlerFile(parent, this, virtualPath, this.BypassAccessCheck);

                return viddlerFile.Video != null ? viddlerFile : null;
            }

            if (string.IsNullOrEmpty(virtualPath))
                return null;

            return this.Previous.GetFile(virtualPath);
        }
    }
}
