﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Text;
using System.IO;
using System.Web;

namespace Nansen.Viddler
{
    public class PostData
    {
        public static string Boundary = "AaB03x";

        public List<PostDataParameter> Params
        {
            get; private set;
        }

        public PostData()
        {
            this.Params = new List<PostDataParameter>();
        }

        private readonly Encoding _encoding = Encoding.UTF8;
        public byte[] GetPostData()
        {
            var formDataStream = new MemoryStream();

            foreach (PostDataParameter p in this.Params)
            {
                if (p is PostDataFile)
                {
                    var fileParameter = p as PostDataFile;
                    string header = string.Format("--{0}\r\nContent-Disposition: form-data; name=\"file\"; filename=\"{1}\";\r\nContent-Type: application/octet-stream\r\n\r\n",
                        Boundary,
                        fileParameter.FileName
                    );

                    formDataStream.Write(_encoding.GetBytes(header), 0, header.Length);
                    this.CopyStream(fileParameter.FileStream, formDataStream);
                }
                else
                {
                    string postData = string.Format("--{0}\r\nContent-Disposition: form-data; name=\"{1}\"\r\n\r\n{2}\r\n",
                    Boundary,
                    p.Name,
                    p.Value);

                    formDataStream.Write(_encoding.GetBytes(postData), 0, postData.Length);
                }
            }

            string footer = "\r\n--" + Boundary + "--\r\n";
            formDataStream.Write(_encoding.GetBytes(footer), 0, footer.Length);

            formDataStream.Position = 0;
            byte[] formData = new byte[formDataStream.Length];
            formDataStream.Read(formData, 0, formData.Length);
            formDataStream.Close();

            return formData;
        }

        private void CopyStream(Stream input, Stream output)
        {
            int bufferSize = 4096;
            var buffer = new byte[bufferSize];
            while (true)
            {
                int read = input.Read(buffer, 0, buffer.Length);
                if (read <= 0)
                {
                    return;
                }

                output.Write(buffer, 0, read);
            }
        }
    }
}
