﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EPiServer.Core;
using EPiServer.Web.PropertyControls;
using System.Web.UI.WebControls;
using System.Xml.Serialization;
using System.IO;
using EPiServer.ClientScript;
using Nansen.Viddler.Properties;

namespace Nansen.Viddler.SpecializedProperties
{
    /// <summary>
    /// TODO: Jävligt glassigt UI (se www.viddler.com > embed)
    /// </summary>
    public class PropertyViddlerVideoControl : PropertyDataControl
    {
        private DropDownList _videoSelector;
        private RadioButtonList _sizePresetSelector;

        public override void ApplyEditChanges()
        {
            if (!string.IsNullOrEmpty(_videoSelector.SelectedValue))
            {
                var videoData = new PropertyViddlerVideoData
                                    {
                                        Autoplay = false,
                                        DisableBranding = false,
                                        SizePreset = _sizePresetSelector.SelectedValue,
                                        Id = _videoSelector.SelectedValue
                                    };

                string xmlString = PropertyViddlerVideoData.SerializeToString(videoData);

                base.SetValue(xmlString);
            }
            else
            {
                base.SetValue(null);
            }
        }

        public override void CreateEditControls()
        {
            _videoSelector = GetVideoSelector();
            _sizePresetSelector = GetSizePresetSelector();

            this.Controls.Add(_videoSelector);
            this.Controls.Add(_sizePresetSelector);
        }

        private DropDownList GetVideoSelector()
        {
            var viddler = new Viddler();

            var videoSelector = new DropDownList();
            videoSelector.DataTextField = "Title";
            videoSelector.DataValueField = "Id";

            videoSelector.DataSource = viddler.GetVideos();
            videoSelector.DataBind();

            videoSelector.Items.Insert(0, new ListItem());

            if (this.PropertyData.Value != null)
            {
                PropertyViddlerVideoData videoData = PropertyViddlerVideoData.SerializeFromString(this.PropertyData.Value as string);
                videoSelector.SelectedValue = videoData.Id;
            }

            return videoSelector;
        }

        private RadioButtonList GetSizePresetSelector()
        {
            var sizePresetSelector = new RadioButtonList();

            foreach(string preset in Settings.Default.SizePresets.Split('|'))
            {
                string[] presetSettings = preset.Split(';');

                string name = presetSettings[0];
                int width = int.Parse(presetSettings[1]);
                int height = int.Parse(presetSettings[2]);
                
                sizePresetSelector.Items.Add(new ListItem(string.Format("{0} ({1}x{2})", name, width, height), name));
            }

            if (this.PropertyData.Value != null)
            {
                PropertyViddlerVideoData videoData = PropertyViddlerVideoData.SerializeFromString(this.PropertyData.Value as string);
                if (!string.IsNullOrEmpty(videoData.SizePreset))
                {
                    try
                    {
                        sizePresetSelector.SelectedValue = videoData.SizePreset;
                    }
                    catch (ArgumentOutOfRangeException)
                    {
                        // Preset has been renamed or deleted
                        sizePresetSelector.SelectedIndex = 0;
                    }
                }
            }
            else
            {
                sizePresetSelector.SelectedIndex = 0;                
            }

            return sizePresetSelector;
        }
    }
}
