﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;
using System.Net;

namespace Nansen.Viddler
{
    public class ViddlerVideo
    {
        public ViddlerVideo(string xmlString)
        {
            XDocument doc = XDocument.Parse(xmlString);

            this.Author = GetStringValue(doc.Descendants("video").Elements("author"));
            this.Id = GetStringValue(doc.Descendants("video").Elements("id"));
            this.Title = GetStringValue(doc.Descendants("video").Elements("title"));
            this.Url = GetStringValue(doc.Descendants("video").Elements("url"));
            this.ThumbnailUrl = GetStringValue(doc.Descendants("video").Elements("thumbnail_url"));
            this.PermanentLinkUrl = GetStringValue(doc.Descendants("video").Elements("permalink"));
            this.Description = GetStringValue(doc.Descendants("video").Elements("description"));
            this.Width = GetStringValue(doc.Descendants("video").Elements("width"));
            this.Height = GetStringValue(doc.Descendants("video").Elements("height"));

            this.Length = GetIntegerValue(doc.Descendants("video").Elements("length_seconds"));
            this.Views = GetIntegerValue(doc.Descendants("video").Elements("view_count"));
            this.Comments = GetIntegerValue(doc.Descendants("video").Elements("comment_count"));

            var uploadedElement = doc.Descendants("video").Elements("upload_time");
            this.Uploaded = uploadedElement.ToList().Count > 0 &&
                !string.IsNullOrEmpty(uploadedElement.Single().Value) ?
                this.FromUnixEpoch(double.Parse(uploadedElement.Single().Value)) :
                DateTime.MinValue;
            
            var updatedElement = doc.Descendants("video").Elements("update_time");
            this.Updated = updatedElement.ToList().Count > 0 &&
                !string.IsNullOrEmpty(updatedElement.Single().Value) ?
                this.FromUnixEpoch(double.Parse(updatedElement.Single().Value)) : 
                DateTime.MinValue;

            if (doc.Descendants("video").Elements("tags").ToList().Count > 0)
            {
                this.Tags = doc.Descendants("video").Elements("tags").Elements("global").Select(e => e.Value);
            }
            else
            {
                this.Tags = new List<string>();
            }
        }

        private string GetStringValue(IEnumerable<XElement> elements)
        {
            return elements.ToList().Count > 0 ? elements.Single().Value : string.Empty;
        }

        private int GetIntegerValue(IEnumerable<XElement> elements)
        {
            int value = 0;
            if(elements.ToList().Count > 0)
            {
                int.TryParse(elements.Single().Value, out value);
            }

            return value;
        }

        public string Id { get; set; }
        public string Author { get; set; }
        public string Title { get; set; }
        public int Length { get; set; }
        public string Width { get; set; }
        public string Height { get; set; }
        public string Description { get; set; }
        public int Views { get; set; }
        public int Comments { get; set; }
        public IEnumerable<string> Tags { get; set; }
        public string Url { get; set; }
        public string ThumbnailUrl { get; set; }
        public string PermanentLinkUrl { get; set; }

        public DateTime Uploaded { get; set; }
        public DateTime Updated { get; set; }

        private static readonly DateTime unixEpoch = new DateTime(1970, 1, 1, 0, 0, 0, 0, DateTimeKind.Utc);
        private DateTime FromUnixEpoch(double ticks)
        {
            return unixEpoch.AddSeconds(ticks / 1000);
        }

    }
}
